package java.beans;

import sun.reflect.misc.ReflectUtil;
import java.lang.ref.WeakReference;
import java.lang.reflect.Method;
import java.lang.ref.SoftReference;

final class MethodRef
{
    private String signature;
    private SoftReference<Method> methodRef;
    private WeakReference<Class<?>> typeRef;
    
    MethodRef() {
    }
    
    void set(Method referent) {
        if (referent == null) {
            this.signature = null;
            this.methodRef = null;
            this.typeRef = null;
        }
        else {
            this.signature = referent.toGenericString();
            this.methodRef = new SoftReference(referent);
            this.typeRef = new WeakReference(referent.getDeclaringClass());
        }
    }
    
    boolean isSet() {
        return this.methodRef != null;
    }
    
    Method get() {
        if (this.methodRef == null) {
            return null;
        }
        Method find = (Method)this.methodRef.get();
        if (find == null) {
            find = find((Class)this.typeRef.get(), this.signature);
            if (find == null) {
                this.signature = null;
                this.methodRef = null;
                this.typeRef = null;
                return null;
            }
            this.methodRef = new SoftReference(find);
        }
        return ReflectUtil.isPackageAccessible(find.getDeclaringClass()) ? find : null;
    }
    
    private static Method find(Class<?> clazz, String anObject) {
        if (clazz != null) {
            Method[] methods = clazz.getMethods();
            int length = methods.length;
            int i = 0;
            while (i < length) {
                Method method = methods[i];
                if (clazz.equals(method.getDeclaringClass()) && method.toGenericString().equals(anObject)) {
                    return method;
                }
                ++i;
            }
        }
        return null;
    }
}
